<?php
/**
 * montonio/module-hire-purchase
 *
 * @category   montonio
 * @package    module-hire-purchase
 * @copyright  Copyright (c) 2020 Montonio Finance OÜ  (https://montonio.com/)
 * @license    https://www.gnu.org/licenses/gpl-3.0.txt
 */
namespace Montonio\Hirepurchase\Gateway\HirePurchase\Response;

use Magento\Payment\Gateway\Data\PaymentDataObjectInterface;
use Magento\Payment\Gateway\Response\HandlerInterface;

/**
 * @inheritdoc
 */
class StatusResponseHandler implements HandlerInterface
{

    /**
     *
     * @param \Magento\Framework\Stdlib\DateTime\TimezoneInterface $timezoneInterface
     * @param \Magento\Payment\Gateway\Data\PaymentDataObjectFactory $paymentDataObjectFactory
     * @param \Magento\Sales\Api\OrderRepositoryInterface $orderRepository
     * @param \Magento\Framework\Api\SearchCriteriaBuilder $searchCriteriaBuilder
     * @param \Magento\Sales\Model\Order\Email\Sender\OrderSender $orderSender
     * @param \Montonio\Hirepurchase\Gateway\Hirepurchase\Config\Config $config
     * @param \Psr\Log\LoggerInterface $logger
     */
    public function __construct(
        \Magento\Framework\Stdlib\DateTime\TimezoneInterface $timezoneInterface,
        \Magento\Payment\Gateway\Data\PaymentDataObjectFactory $paymentDataObjectFactory,
        \Magento\Sales\Api\OrderRepositoryInterface $orderRepository,
        \Magento\Framework\Api\SearchCriteriaBuilder $searchCriteriaBuilder,
        \Magento\Sales\Model\Order\Email\Sender\OrderSender $orderSender,
        \Montonio\Hirepurchase\Gateway\Hirepurchase\Config\Config $config,
        \Psr\Log\LoggerInterface $logger
    ) {
        $this->timezoneInterface = $timezoneInterface;
        $this->paymentDataObjectFactory = $paymentDataObjectFactory;
        $this->orderRepository = $orderRepository;
        $this->searchCriteriaBuilder = $searchCriteriaBuilder;
        $this->orderSender = $orderSender;
        $this->config = $config;
        $this->logger = $logger;
    }

    /**
     * Handles gateway repsonse
     *
     * @param array $handlingSubject
     * @param array $response
     * @return void
     */
    public function handle(array $handlingSubject, array $response)
    {

        if (!isset($handlingSubject['payment'])) {
            if (!$handlingSubject['payment'] instanceof PaymentDataObjectInterface &&
                is_object($handlingSubject['payment'])
            ) {
                try {
                    $handlingSubject['payment'] = $this->paymentDataObjectFactory->create($handlingSubject['payment']);
                } catch (\Exception $e) {
                    throw new \InvalidArgumentException(__($e->getMessage()));
                }
            } else {
                throw new \InvalidArgumentException(__('Payment data object should be provided'));
            }
        }

        if (!isset($response['status'])) {
            throw new \InvalidArgumentException(
                __('Gateway Application Validation Request should respond with Status')
            );
        }

        $response['request_datetime'] = $this->timezoneInterface
                ->formatDateTime(
                    $this->timezoneInterface->date(),
                    \IntlDateFormatter::MEDIUM,
                    \IntlDateFormatter::MEDIUM
                );

        $payment = is_object($handlingSubject['payment']->getPayment()) ?
                $handlingSubject['payment']->getPayment() :
                $handlingSubject['payment'];

        $payment->setAdditionalInformation('gateway_response', $response);

        //update order status to processing if status == signed
        if ($response['status'] == 'signed') {

            try {
                //load order and set state we need to laod it again since
                // in admin it is not a real object
                $order = $this->getOrderByIncrementId(
                    $payment->getOrder()->getData('increment_id')
                );

                $orderStatus = \Magento\Sales\Model\Order::STATE_PROCESSING;

                if ($orderStatus &&
                    $order->getState() == \Magento\Sales\Model\Order::STATE_NEW &&
                    $order->getStatus() == 'pending'
                ) {

                    //this is needed for registry object
                    //in admin to reflect the state on first load
                    $payment->getOrder()
                        ->setState($orderStatus)
                        ->setStatus($orderStatus);

                    $order
                        ->setState($orderStatus)
                        ->setStatus($orderStatus)
                        ->addStatusHistoryComment(
                            __(
                                'Order status and state updated to %1 by %2 payment gateway response',
                                $orderStatus,
                                $payment->getMethod()
                            )
                        );

                    $controlEmail = $this->config->getValue('order_email_after');
                    if ($controlEmail && $order->getEmailSent() === null) {
                        try {
                            $this->orderSender->send($order);
                            $order
                                ->addStatusHistoryComment(
                                    __('Order confirmation e-mail sent to customer')
                                );
                        } catch (\Exception $e) {
                            $this->logger->critical($e);
                        }
                    }

                    $this->orderRepository->save($order);

                }
            } catch (\Exception $e) {
                //processing failed, let it fail silently as actual payment succeeded
                $this->logger->critical($e);
            }
        }
    }

    /**
     * @param string $incrementId
     * @return \Magento\Sales\Api\Data\OrderInterface $order
     */
    private function getOrderByIncrementId($incrementId)
    {
        $this->searchCriteriaBuilder->addFilter('increment_id', $incrementId);

        $order = $this->orderRepository->getList(
            $this->searchCriteriaBuilder->create()
        );

        return $order->getFirstItem();
    }
}
