<?php
/**
 * montonio/module-hire-purchase
 *
 * @category   montonio
 * @package    module-hire-purchase
 * @copyright  Copyright (c) 2020 Montonio Finance OÜ  (https://montonio.com/)
 * @license    https://www.gnu.org/licenses/gpl-3.0.txt
 */
namespace Montonio\Hirepurchase\Gateway\Payments\Command;

/**
 * Class AspspsCommand for requesting available methods from gateway
 */
class AspspsCommand implements \Magento\Payment\Gateway\CommandInterface
{
    /**
     *
     * @param \Montonio\Hirepurchase\Gateway\Payments\Config\Config $config
     * @param \Magento\Framework\Stdlib\DateTime\TimezoneInterface $timezoneInterface
     * @param \Magento\Framework\UrlInterface $urlInterface
     * @param \Magento\Framework\HTTP\ClientFactory $httpClientFactory
     * @param \Magento\Payment\Gateway\Http\TransferBuilder $transferBuilder
     * @param \Psr\Log\LoggerInterface $logger
     */
    public function __construct(
        \Montonio\Hirepurchase\Gateway\Payments\Config\Config $config,
        \Magento\Framework\Stdlib\DateTime\TimezoneInterface $timezoneInterface,
        \Magento\Framework\UrlInterface $urlInterface,
        \Magento\Framework\HTTP\ClientFactory $httpClientFactory,
        \Magento\Payment\Gateway\Http\TransferBuilder $transferBuilder,
        \Psr\Log\LoggerInterface $logger
    ) {
            $this->config = $config;
            $this->timezoneInterface = $timezoneInterface;
            $this->urlInterface = $urlInterface;
            $this->httpClientFactory = $httpClientFactory;
            $this->transferBuilder = $transferBuilder;
            $this->logger = $logger;
    }

    /**
     * return request data
     *
     * @param array $commandSubject
     *
     * @return array|Command\ResultInterface
     */
    public function execute(array $commandSubject)
    {
        $data = $this->buildRequestData();
        return $this->requestData($data);
    }

    protected function buildRequestData()
    {

        $time = $this->timezoneInterface->date()->format('U');
        $environment = $this->config->getValue('environment');
        $data = [
            'access_key' => $this->config->getValue($environment.'_public_key'),
            'iat'        => $time,
            'exp'        => $time + (60 * 10)
        ];

        return $data;
    }

    protected function requestData($data)
    {

        $environment = $this->config->getValue('environment');

        $bearerJwtToken = $this->config->generateSignature(
            $data,
            $this->config->getValue($environment.'_private_key')
        );

        $apiUrl = $this->config->getValue($environment.'_api_url');
        $apiEndpoint = $this->config->getValue('listofbanks_api_endpoint');

        $transferObject = $this->transferBuilder
            ->setUri($apiUrl.$apiEndpoint)
            ->setHeaders([
                    'Authorization' => 'Bearer ' . $bearerJwtToken
                ])
            ->setMethod('GET')
            ->build();

        try {

            $client = $this->getCurlClient();
            $client->setHeaders($transferObject->getHeaders());

            if ($transferObject->getMethod() === 'GET') {
                $client->get($transferObject->getUri());
            }

            $responseBody = $client->getBody();
            $response = json_decode($client->getBody(), true);

            if (isset($response['statusCode']) && $response['statusCode'] !== '200') {
                $message = __('aspsps request status %1.', $response['statusCode']);
                throw new \Magento\Framework\Exception\LocalizedException($message);
            }

        } catch (\Exception $e) {
            $this->logger->critical($e);
            throw new \Magento\Framework\Exception\LocalizedException(__($e->getMessage()));
        }

        return $response;
    }

    /**
     * @return Curl
     */
    protected function getCurlClient()
    {
        return $this->httpClientFactory->create();
    }
}
